-- RBT-TEST - Проверка работы индексов
-- Проверка сохранения атрибутов безопасности при использовании индексов
--

\i 'support/init.sql'
-- 
-- Тестируемые методы доступа:
-- Seq Scan -- читается вся таблица.
-- Tid Scan -- используется только при выборе единичной записи таблицы по ее TID.
-- Index Scan -- используется индекс для условий WHERE, читает таблицу при отборе строк.
-- Bitmap Index Scan -- сначала Index Scan, затем контроль выборки по таблице. Эффективно для большого количества строк.
-- Index Only Scan -- самый быстрый. Читается только индекс.
--
-- Создаем таблицу с составным ключом и защищенными строками
CREATE TABLE macindex (
	id   INTEGER,
	num  NUMERIC,
	data TEXT,

	PRIMARY KEY (id, num)
) WITH (MACS=TRUE);
GRANT ALL ON macindex TO PUBLIC;
ALTER TABLE macindex SET MAC TO NULL;
--
-- Создаем таблицу с составным ключом, по мандатной метке и защищенными строками
CREATE TABLE macindex1 (
	id   INTEGER,
	num  NUMERIC,
	data TEXT,

	PRIMARY KEY (maclabel, id)
) WITH (MACS=TRUE);
GRANT ALL ON macindex1 TO PUBLIC;
ALTER TABLE macindex1 SET MAC TO NULL;
--
-- Создаем таблицу с простым ключом и защищенными строками
CREATE TABLE macindex2 (
	id   INTEGER PRIMARY KEY,
	num  NUMERIC,
	data TEXT
) WITH (MACS=TRUE);
GRANT ALL ON macindex2 TO PUBLIC;
ALTER TABLE macindex2 SET MAC TO NULL;
--
-- Создаем расширение pg_hint_plan
CREATE EXTENSION pg_hint_plan;
VACUUM FULL;
--
-- Подключаемся как пользователь u_2_11 с меткой {2,3}
\connect - u_2_11
--
-- И вставляем записи во все таблицы
INSERT INTO macindex VALUES  (1, 1, '{2,3}');
INSERT INTO macindex1 VALUES (1, 1, '{2,3}');
INSERT INTO macindex2 VALUES (1, 1, '{2,3}');
--
-- Проверяем, чтобы записи в таблицах присутствуют
SELECT * FROM macindex;
SELECT * FROM macindex1;
SELECT * FROM macindex2;
--
-- Подключаемся как пользователь u_0_00 с меткой {0,0}
\connect - u_0_00
--
-- Проверяем, чтобы записи с высокой меткой были не видны
-- Ожидается, что выборки будут пустыми
--
-- ====================================================================================================
-- Seq Scan
-- ====================================================================================================
--
EXPLAIN (COSTS FALSE, VERBOSE) /*+ SeqScan(macindex) */ SELECT * FROM macindex;
SELECT * FROM macindex;
--
EXPLAIN (COSTS FALSE, VERBOSE) /*+ SeqScan(macindex) */ SELECT * FROM macindex WHERE id < 3;
SELECT * FROM macindex WHERE id < 3;
--
EXPLAIN (COSTS FALSE, VERBOSE) /*+ SeqScan(macindex) */ SELECT * FROM macindex WHERE data LIKE '{?,?}';
SELECT * FROM macindex WHERE data LIKE '{?,?}';
--
EXPLAIN (COSTS FALSE, VERBOSE) /*+ SeqScan(macindex) */ SELECT maclabel,* FROM macindex;
SELECT maclabel,* FROM macindex;
--
EXPLAIN (COSTS FALSE, VERBOSE) /*+ SeqScan(macindex) */ SELECT maclabel,* FROM macindex WHERE id < 3;
SELECT maclabel,* FROM macindex WHERE id < 3;
--
-- ====================================================================================================
-- Tid Scan
-- ====================================================================================================
--
EXPLAIN (COSTS FALSE, VERBOSE) /*+ TidScan(macindex) */ SELECT * FROM macindex WHERE ctid  = any(array(SELECT ctid FROM macindex LIMIT 10));
SELECT * FROM macindex where ctid  = any(array(SELECT ctid FROM macindex LIMIT 10));
--
-- ====================================================================================================
-- Index Scan
-- ====================================================================================================
--
EXPLAIN (COSTS FALSE, VERBOSE) /*+ IndexScan(macindex1) */ SELECT * FROM macindex1 WHERE id < 3;
SELECT * FROM macindex1 WHERE id < 3;
--
EXPLAIN (COSTS FALSE, VERBOSE) /*+ IndexScan(macindex1) */ SELECT maclabel,* FROM macindex1 WHERE id < 3;
SELECT maclabel,* FROM macindex1 WHERE id < 3;
--
EXPLAIN (COSTS FALSE, VERBOSE) /*+ IndexScan(macindex2) */ SELECT * FROM macindex2 WHERE id < 3;
SELECT * FROM macindex2 WHERE id < 3;
--
EXPLAIN (COSTS FALSE, VERBOSE) /*+ IndexScan(macindex2) */ SELECT maclabel,* FROM macindex2 WHERE id < 3;
SELECT maclabel,* FROM macindex2 WHERE id < 3;
--
-- ====================================================================================================
-- Index Only Scan
-- ====================================================================================================
--
EXPLAIN (COSTS FALSE, VERBOSE) /*+ IndexOnlyScan(macindex1) */ SELECT count(*) FROM macindex1;
SELECT count(*) FROM macindex1;
--
EXPLAIN (COSTS FALSE, VERBOSE) /*+ IndexOnlyScan(macindex1) */ SELECT maclabel,id FROM macindex1 WHERE id < 3;
SELECT maclabel,id FROM macindex1 WHERE id < 3;
--
EXPLAIN (COSTS FALSE, VERBOSE) /*+ IndexOnlyScan(macindex2) */ SELECT count(*) FROM macindex2;
SELECT count(*) FROM macindex2;
--
EXPLAIN (COSTS FALSE, VERBOSE) /*+ IndexOnlyScan(macindex2) */ SELECT id FROM macindex2 WHERE id < 3;
SELECT id FROM macindex2 WHERE id < 3;
--
-- ====================================================================================================
-- Bitmap Only Scan
-- ====================================================================================================
--
EXPLAIN (COSTS FALSE, VERBOSE) /*+ BitMapScan(macindex1) */ SELECT * FROM macindex1 WHERE id BETWEEN 0 AND 5;
SELECT * FROM macindex1 WHERE id BETWEEN 0 AND 5;
--
EXPLAIN (COSTS FALSE, VERBOSE) /*+ BitMapScan(macindex2) */ SELECT * FROM macindex2 WHERE id BETWEEN 0 AND 5;
SELECT * FROM macindex2 WHERE id BETWEEN 0 AND 5;
--
-- ====================================================================================================
-- Проверяем методы соединения таблиц
-- ====================================================================================================
--
-- Тестируются методы соединения таблиц:
-- NestLoop
-- HashJoin
-- MergeJoin
--
EXPLAIN (COSTS FALSE, VERBOSE) /*+ NestLoop(macindex1 macindex2) */ SELECT mi.id, mi.data, mi1.id, mi1.data, mi2.id, mi2.data
FROM macindex mi LEFT JOIN macindex1 mi1 ON mi1.id = mi.id
LEFT JOIN macindex2 mi2 ON mi2.id = mi1.id;
SELECT mi.id, mi.data, mi1.id, mi1.data, mi2.id, mi2.data
FROM macindex mi LEFT JOIN macindex1 mi1 ON mi1.id = mi.id
LEFT JOIN macindex2 mi2 ON mi2.id = mi1.id;
--
EXPLAIN (COSTS FALSE, VERBOSE) /*+ HashJoin(macindex1 macindex2) */ SELECT * FROM macindex1 mi1 JOIN macindex2 mi2 ON mi1.id = mi2.id;
SELECT * FROM macindex1 mi1 JOIN macindex2 mi2 ON mi1.id = mi2.id;
--
--
-- ====================================================================================================
-- Различные проверки индексов
-- ====================================================================================================
--
-- Пытаемся вставить те же данные
-- В таблицу с составным ключом
-- Ожидается ошибка нарушение ограничения
INSERT INTO macindex VALUES (1, 1, '{0,0}');
--
-- В таблицу с составным ключом, по мандатной метке
-- Ожидается успех
INSERT INTO macindex1 VALUES (1, 1, '{0,0}');
--
-- В таблицу с простым ключом
-- Ожидается ошибка нарушение ограничения
INSERT INTO macindex2 VALUES (1, 1, '{0,0}');
--
-- Проверяем таблицу с составным ключом
-- Ожидается пустая выборка
SELECT * FROM macindex;
SELECT * FROM macindex WHERE id < 3;
--
-- Проверяем таблицу с составным ключом, по мандатной метке
-- Ожидается одна строка пользователя u_0_00 с меткой {0,0}
SELECT * FROM macindex1;
SELECT * FROM macindex1 WHERE id < 3;
--
-- Проверяем таблицу с простым ключом
-- Ожидается пустая выборка
SELECT * FROM macindex2;
SELECT * FROM macindex2 WHERE id < 3;
--
-- Пытаемся вставить новые данные
-- Ожидается успех во всех случаях
INSERT INTO macindex VALUES (2, 2, '{0,0}');
INSERT INTO macindex1 VALUES (2, 2, '{0,0}');
INSERT INTO macindex2 VALUES (2, 2, '{0,0}');
--
-- Проверяем таблицу с составным ключом
-- Ожидается одна запись пользователя u_0_00 с меткой {0,0}
SELECT * FROM macindex;
SELECT * FROM macindex WHERE id < 3;
--
-- Проверяем таблицу с составным ключом, по мандатной метке
-- Ожидается две записи пользователя u_0_00 с меткой {0,0}
SELECT * FROM macindex1;
SELECT * FROM macindex1 WHERE id < 3;
--
-- Проверяем таблицу с простым ключом
-- Ожидается одна запись пользователя u_0_00 с меткой {0,0}
SELECT * FROM macindex2;
SELECT * FROM macindex2 WHERE id < 3;
--
-- Ожидается 1 запись
SELECT mi.id, mi.data, mi1.id, mi1.data, mi2.id, mi2.data
FROM macindex mi LEFT JOIN macindex1 mi1 on mi1.id = mi.id
LEFT JOIN macindex2 mi2 on mi2.id=mi1.id;
--
-- Ожидается 1 запись
SELECT mi.id, mi.data
FROM macindex mi
WHERE EXISTS(select * from macindex1 mi1 where mi1.id = mi.id);
--
-- Подключаемся как пользователь u_2_11 с меткой {2,3}
\connect - dbsa
--
-- Проверяем, что видны все записи, сделанные обоими пользователями
SELECT * FROM macindex;
SELECT * FROM macindex WHERE id < 3;
SELECT * FROM macindex1;
SELECT * FROM macindex1 WHERE id < 3;
SELECT * FROM macindex2;
SELECT * FROM macindex2 WHERE id < 3;
--
-- Ожидается 3 записи
SELECT mi.id, mi.data, mi1.id, mi1.data, mi2.id, mi2.data
FROM macindex mi LEFT JOIN macindex1 mi1 on mi1.id = mi.id
LEFT JOIN macindex2 mi2 on mi2.id=mi1.id;
--
-- Ожидается 2 записи
SELECT mi.id, mi.data
FROM macindex mi
WHERE EXISTS(select * from macindex1 mi1 where mi1.id = mi.id); 
--
\i 'support/done.sql'